/*
**  Gen.c
**  7 October 1997
*/
/*
## NAME 
## 	Gen - Construct Schrier-Sims representation of group
## 
## SYNOPSIS
## 	Gen [fname]
## 
## DESCRIPTION
## 	Gen reads  permutations from stdin or from
##	the file <fname>. The Schrier-Sims 
##	representation of the group generated by these
##	permutations with respect to the base is 
##	written to stdout.
##
## INPUT FORMAT
##	Ngen deg
##	(permutation_1 in cycle form)
##	(permutation_2 in cycle form)
##		. . .
##	(permutation_Ngen in cycle form)
##
##	where	Ngen	= the number of generators
##		deg	= the degree of the 
##				representation
*/

/***************************************************/
#include<stdio.h>
#include<stdlib.h>
#include "base.h"
#include "grouplib.h"
/***************************************************/
void usage(char s[])
{
  fprintf(stderr,"\nUsage: %s [fname]\n",s);
  exit(1);
}
/***************************************************/

int main(int ac, char *av[])
{
 FILE *in;
 int i,Ngen,deg;
 universe  U;
 perm x;
 perm base;
 group G;
 permlist Gamma;
setbuf(stdout,0);
 in=stdin;
 switch(ac)
 {
  case 1:
   in=stdin;
  break;

  case 2:
   if( (in=fopen(av[1],"r"))==NULL )
   {
    fprintf(stderr,"%s cannot open %s\n",av[0],av[1]);
    exit(1);
   }
  break;

  default:
   usage(av[0]);
  break;
 }
 fscanf(in,"%d %d",&Ngen,&deg);
 U=NewUniverse(deg);
 base=NewPerm(U);
 G=NewGroup(U,base);
 Gamma=NewPermList(U,Ngen);
 x=NewPerm(U);
 for(i=0;i<Ngen;i++)
 {
  Gamma[i]=NewPerm(U);
  ReadPerm(in,Gamma[i]);
 }
 Gen(Ngen,Gamma,G);
 OutGroup(stdout,G);

 return(0);
}
